@@ -1,18 +1,20 @@
-# $Id: CheckLib.pm,v 1.21 2008/03/04 21:49:20 drhyde Exp $
+# $Id: CheckLib.pm,v 1.25 2008/10/27 12:16:23 drhyde Exp $
 
 package Devel::CheckLib;
 
+use 5.00405; #postfix foreach
 use strict;
 use vars qw($VERSION @ISA @EXPORT);
-$VERSION = '0.4';
+$VERSION = '0.9';
 use Config;
+use Text::ParseWords 'quotewords';
 
 use File::Spec;
 use File::Temp;
 
 require Exporter;
 @ISA = qw(Exporter);
-@EXPORT = qw(assert_lib check_lib_or_exit);
+@EXPORT = qw(assert_lib check_lib_or_exit check_lib);
 
 # localising prevents the warningness leaking out of this module
 local $^W = 1;    # use warnings is a 5.6-ism
@@ -30,8 +32,6 @@ library and its headers are available.
 
 =head1 SYNOPSIS
 
-    # in a Makefile.PL or Build.PL
-    use lib qw(inc);
     use Devel::CheckLib;
 
     check_lib_or_exit( lib => 'jpeg', header => 'jpeglib.h' );
@@ -40,20 +40,44 @@ library and its headers are available.
     # or prompt for path to library and then do this:
     check_lib_or_exit( lib => 'jpeg', libpath => $additional_path );
 
+=head1 USING IT IN Makefile.PL or Build.PL
+
+If you want to use this from Makefile.PL or Build.PL, do
+not simply copy the module into your distribution as this may cause
+problems when PAUSE and search.cpan.org index the distro.  Instead, use
+the use-devel-checklib script.
+
 =head1 HOW IT WORKS
 
 You pass named parameters to a function, describing to it how to build
 and link to the libraries.
 
-It works by trying to compile this:
+It works by trying to compile some code - which defaults to this:
 
     int main(void) { return 0; }
 
 and linking it to the specified libraries.  If something pops out the end
-which looks executable, then we know that it worked.  That tiny program is
+which looks executable, it gets executed, and if main() returns 0 we know
+that it worked.  That tiny program is
 built once for each library that you specify, and (without linking) once
 for each header file.
 
+If you want to check for the presence of particular functions in a
+library, or even that those functions return particular results, then
+you can pass your own function body for main() thus:
+
+    check_lib_or_exit(
+        function => 'foo();if(libversion() > 5) return 0; else return 1;'
+        incpath  => ...
+        libpath  => ...
+        lib      => ...
+        header   => ...
+    );
+
+In that case, it will fail to build if either foo() or libversion() don't
+exist, and main() will return the wrong value if libversion()'s return
+value isn't what you want.
+
 =head1 FUNCTIONS
 
 All of these take the same named parameters and are exported by default.
@@ -88,6 +112,8 @@ representing additional paths to search for libraries.
 a C<ExtUtils::MakeMaker>-style space-seperated list of
 libraries (each preceded by '-l') and directories (preceded by '-L').
 
+This can also be supplied on the command-line.
+
 =back
 
 And libraries are no use without header files, so ...
@@ -109,6 +135,8 @@ representing additional paths to search for headers.
 a C<ExtUtils::MakeMaker>-style space-seperated list of
 incpaths, each preceded by '-I'.
 
+This can also be supplied on the command-line.
+
 =back
 
 =head2 check_lib_or_exit
@@ -124,6 +152,11 @@ causing a CPAN Testers 'FAIL' report.  CPAN Testers should ignore this
 result -- which is what you want if an external library dependency is not
 available.
 
+=head2 check_lib
+
+This behaves exactly the same as C<assert_lib()> except that it is silent,
+returning false instead of dieing, or true otherwise.
+
 =cut
 
 sub check_lib_or_exit {
@@ -134,6 +167,11 @@ sub check_lib_or_exit {
     }
 }
 
+sub check_lib {
+    eval 'assert_lib(@_)';
+    return $@ ? 0 : 1;
+}
+
 sub assert_lib {
     my %args = @_;
     my (@libs, @libpaths, @headers, @incpaths);
@@ -149,14 +187,27 @@ sub assert_lib {
         if $args{incpath};
 
     # work-a-like for Makefile.PL's LIBS and INC arguments
+    # if given as command-line argument, append to %args
+    for my $arg (@ARGV) {
+        for my $mm_attr_key (qw(LIBS INC)) {
+            if (my ($mm_attr_value) = $arg =~ /\A $mm_attr_key = (.*)/x) {
+            # it is tempting to put some \s* into the expression, but the
+            # MM command-line parser only accepts LIBS etc. followed by =,
+            # so we should not be any more lenient with whitespace than that
+                $args{$mm_attr_key} .= " $mm_attr_value";
+            }
+        }
+    }
+
+    # using special form of split to trim whitespace
     if(defined($args{LIBS})) {
-        foreach my $arg (split(/\s+/, $args{LIBS})) {
+        foreach my $arg (split(' ', $args{LIBS})) {
             die("LIBS argument badly-formed: $arg\n") unless($arg =~ /^-l/i);
             push @{$arg =~ /^-l/ ? \@libs : \@libpaths}, substr($arg, 2);
         }
     }
     if(defined($args{INC})) {
-        foreach my $arg (split(/\s+/, $args{INC})) {
+        foreach my $arg (split(' ', $args{INC})) {
             die("INC argument badly-formed: $arg\n") unless($arg =~ /^-I/);
             push @incpaths, substr($arg, 2);
         }
@@ -164,25 +215,43 @@ sub assert_lib {
 
     my @cc = _findcc();
     my @missing;
+    my @wrongresult;
+    my @use_headers;
 
     # first figure out which headers we can't find ...
     for my $header (@headers) {
+        push @use_headers, $header;
         my($ch, $cfile) = File::Temp::tempfile(
             'assertlibXXXXXXXX', SUFFIX => '.c'
         );
-        print $ch qq{#include <$header>\nint main(void) { return 0; }\n};
+        print $ch qq{#include <$_>\n} for @use_headers;
+        print $ch qq{int main(void) { return 0; }\n};
         close($ch);
         my $exefile = File::Temp::mktemp( 'assertlibXXXXXXXX' ) . $Config{_exe};
         my @sys_cmd;
         # FIXME: re-factor - almost identical code later when linking
         if ( $Config{cc} eq 'cl' ) {                 # Microsoft compiler
             require Win32;
-            @sys_cmd = (@cc, $cfile, "/Fe$exefile", (map { '/I'.Win32::GetShortPathName($_) } @incpaths));
+            @sys_cmd = (
+                @cc,
+                $cfile,
+                "/Fe$exefile",
+                (map { '/I'.Win32::GetShortPathName($_) } @incpaths)
+            );
         } elsif($Config{cc} =~ /bcc32(\.exe)?/) {    # Borland
-            @sys_cmd = (@cc, (map { "-I$_" } @incpaths), "-o$exefile", $cfile);
-        } else {                                     # Unix-ish
-                                                     # gcc, Sun, AIX (gcc, cc)
-            @sys_cmd = (@cc, $cfile, (map { "-I$_" } @incpaths), "-o", "$exefile");
+            @sys_cmd = (
+                @cc,
+                (map { "-I$_" } @incpaths),
+                "-o$exefile",
+                $cfile
+            );
+        } else { # Unix-ish: gcc, Sun, AIX (gcc, cc), ...
+            @sys_cmd = (
+                @cc,
+                $cfile,
+                (map { "-I$_" } @incpaths),
+                "-o", "$exefile"
+            );
         }
         warn "# @sys_cmd\n" if $args{debug};
         my $rv = $args{debug} ? system(@sys_cmd) : _quiet_system(@sys_cmd);
@@ -191,11 +260,12 @@ sub assert_lib {
         unlink $cfile;
     } 
 
-    # now do each library in turn with no headers
+    # now do each library in turn with headers
     my($ch, $cfile) = File::Temp::tempfile(
         'assertlibXXXXXXXX', SUFFIX => '.c'
     );
-    print $ch "int main(void) { return 0; }\n";
+    print $ch qq{#include <$_>\n} foreach (@headers);
+    print $ch "int main(void) { ".($args{function} || 'return 0;')." }\n";
     close($ch);
     for my $lib ( @libs ) {
         my $exefile = File::Temp::mktemp( 'assertlibXXXXXXXX' ) . $Config{_exe};
@@ -205,27 +275,48 @@ sub assert_lib {
             my @libpath = map { 
                 q{/libpath:} . Win32::GetShortPathName($_)
             } @libpaths; 
-            @sys_cmd = (@cc, $cfile, "${lib}.lib", "/Fe$exefile", 
-                        "/link", @libpath
+            # this is horribly sensitive to the order of arguments
+            @sys_cmd = (
+                @cc,
+                $cfile,
+                "${lib}.lib",
+                "/Fe$exefile", 
+                (map { '/I'.Win32::GetShortPathName($_) } @incpaths),
+                "/link",
+                (map {'/libpath:'.Win32::GetShortPathName($_)} @libpaths),
             );
         } elsif($Config{cc} eq 'CC/DECC') {          # VMS
         } elsif($Config{cc} =~ /bcc32(\.exe)?/) {    # Borland
-            my @libpath = map { "-L$_" } @libpaths;
-            @sys_cmd = (@cc, "-o$exefile", "-l$lib", @libpath, $cfile);
+            @sys_cmd = (
+                @cc,
+                "-o$exefile",
+                "-l$lib",
+                (map { "-I$_" } @incpaths),
+                (map { "-L$_" } @libpaths),
+                $cfile);
         } else {                                     # Unix-ish
                                                      # gcc, Sun, AIX (gcc, cc)
-            my @libpath = map { "-L$_" } @libpaths;
-            @sys_cmd = (@cc, $cfile,  "-o", "$exefile", "-l$lib", @libpath);
+            @sys_cmd = (
+                @cc,
+                $cfile,
+                "-o", "$exefile",
+                "-l$lib",
+                (map { "-I$_" } @incpaths),
+                (map { "-L$_" } @libpaths)
+            );
         }
         warn "# @sys_cmd\n" if $args{debug};
         my $rv = $args{debug} ? system(@sys_cmd) : _quiet_system(@sys_cmd);
-        push @missing, $lib if $rv != 0 || ! -x $exefile; 
+        push @missing, $lib if $rv != 0 || ! -x $exefile;
+        push @wrongresult, $lib if $rv == 0 && -x $exefile && system(File::Spec->rel2abs($exefile)) != 0; 
         _cleanup_exe($exefile);
     } 
     unlink $cfile;
 
     my $miss_string = join( q{, }, map { qq{'$_'} } @missing );
     die("Can't link/include $miss_string\n") if @missing;
+    my $wrong_string = join( q{, }, map { qq{'$_'} } @wrongresult);
+    die("wrong result: $wrong_string\n") if @wrongresult;
 }
 
 sub _cleanup_exe {
@@ -239,12 +330,14 @@ sub _cleanup_exe {
 }
     
 sub _findcc {
+    my @flags = grep { length } map { quotewords('\s+', 0, $_) }
+        @Config{qw(ccflags ldflags)};
     my @paths = split(/$Config{path_sep}/, $ENV{PATH});
     my @cc = split(/\s+/, $Config{cc});
-    return @cc if -x $cc[0];
+    return (@cc, @flags) if -x $cc[0];
     foreach my $path (@paths) {
         my $compiler = File::Spec->catfile($path, $cc[0]) . $Config{_exe};
-        return ($compiler, @cc[1 .. $#cc]) if -x $compiler;
+        return ($compiler, @cc[1 .. $#cc], @flags) if -x $compiler;
     }
     die("Couldn't find your C compiler\n");
 }
@@ -295,12 +388,16 @@ It has been tested with varying degrees on rigourousness on:
 
 =item IBM's tools on AIX
 
+=item SGI's tools on Irix 6.5
+
 =item Microsoft's tools on Windows
 
 =item MinGW on Windows (with Strawberry Perl)
 
 =item Borland's tools on Windows
 
+=item QNX
+
 =back
 
 =head1 WARNINGS, BUGS and FEEDBACK
@@ -329,10 +426,14 @@ David Cantrell E<lt>david@cantrell.org.ukE<gt>
 
 David Golden E<lt>dagolden@cpan.orgE<gt>
 
+Yasuhiro Matsumoto E<lt>mattn@cpan.orgE<gt>
+
 Thanks to the cpan-testers-discuss mailing list for prompting us to write it
 in the first place;
 
-to Chris Williams for help with Borland support.
+to Chris Williams for help with Borland support;
+
+to Tony Cook for help with Microsoft compiler command-line options
 
 =head1 COPYRIGHT and LICENCE
 
